------------------------------------------------
-- UPC - EETAC - CSD - http://digsys.cat
--
-- The top schematic of the traffic light controller 
-- Strategy: structure of components, logic and signals 
------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY traffic_light_controller	IS  
   Port (
      CLK, CD       : IN	STD_LOGIC;
      Night_Mode    : IN	STD_LOGIC;  
      AG, AY, AR 	: OUT	STD_LOGIC;    
      BG, BY, BR   	: OUT	STD_LOGIC      
   );
END traffic_light_controller ;

ARCHITECTURE schematic OF traffic_light_controller IS  
-- Components

	COMPONENT traffic_light_FSM IS  
	   Port (
	      CLK, CD       : IN	STD_LOGIC;
	      NM 		    : IN	STD_LOGIC;  
	      RT_Flag	    : IN	STD_LOGIC;      
	      AG, AY, AR 	: OUT	STD_LOGIC;    
	      BG, BY, BR   	: OUT	STD_LOGIC;      
	      Sel_night		: OUT	STD_LOGIC
	   );
	END COMPONENT;

	COMPONENT Dual_MUX2 IS  
		PORT (
			Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR (1 downto 0);  -- 2 bits per channel
			S			:	IN 	STD_LOGIC;						-- to select 2 channels
			E			: 	IN 	STD_LOGIC; 						-- enable input
			Y		   	: 	OUT STD_LOGIC_VECTOR (1 downto 0)
	  	);
	END COMPONENT;
       
	COMPONENT counter_mod12 IS  
		Port (
		  	CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			CE 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC_VECTOR(3 DOWNTO  0);
			TC12	: OUT	STD_LOGIC
		);
	END COMPONENT;
	
-- Signals for connecting components together (just the internal wires)
	SIGNAL Sel_night		:	std_logic;
	SIGNAL CLK_2Hz_SQ		: 	std_logic;
	SIGNAL AY_n, BY_n		: 	std_logic;
	SIGNAL Real_Time_Flag	: 	std_logic;		
	SIGNAL Q				: 	std_logic_vector(3 downto 0);	

	BEGIN  
-- Instantiation of components
	Chip1 : traffic_light_FSM
		PORT MAP (
			CLK		=> CLK,
			CD		=> CD,
			NM 		=> Night_Mode,   
			RT_Flag	=> Real_Time_Flag,
			AG		=> AG,
			AY		=> AY_n,
			AR		=> AR,
			BG		=> BG,
			BY		=> BY_n,
			BR		=> BR,
			Sel_night	=> Sel_night
		);

	Chip2	: counter_mod12
		PORT MAP (
			CLK				=> CLK,
			CD 	   			=> CD,
			CE				=> '1',
			Q(3 downto 1) 	=> Q(3 downto 1),	  
			Q(0) 			=> CLK_2Hz_SQ,
			TC12			=> Real_Time_Flag							  
		);

	Chip3	: Dual_MUX2
		PORT MAP (
			E			=> '1',
			S			=> Sel_night,
			Ch0(0)		=> AY_n,
			Ch0(1) 		=> BY_n,
			Ch1(0)		=> CLK_2Hz_SQ,
			Ch1(1)		=> CLK_2Hz_SQ,	  
			Y(0)		=> AY,
			Y(1)		=> BY
		);

	-- Other connections and logic between components

END schematic ;
